#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Publish a tagged plot along the corresponding research data.

Export the tagged figure to a directory that also contains the raw data where
the plot is based on. Additionally, the script that produced the plot will be
copied to the destination directory.

Functions:
    publish(path-like, path-like, figure, string, string) -> None
"""

import os
import shutil
import sys
import warnings
from plotid.save_plot import save_plot


def publish(src_datapath, dst_path, figure, plot_name,
            data_storage='individual'):
    """
    Save plot, data and measuring script.

    Parameters
    ----------
    src_datapath : string
        Path to data that should be published.
    dst_path : string
        Path to the destination directory.
    figure : figure object
        Figure that was tagged and now should be saved as picture.
    plot_name: string
        Name for the exported plot.
    data_storage: string
        Method how the data should be stored. Available options:
            centralized: The raw data will copied only once. All other plots
                will reference this data via sym link.
            individual: The complete raw data will be copied to a folder for
                every plot, respectively. This is the default value.

    Returns
    -------
    None.

    """
    # Check if source directory exists
    if not os.path.exists(src_datapath):
        raise FileNotFoundError('The specified source directory '
                                'does not exist.')

    # Check if destination directory is allowed path
    dst_path_head, dst_dirname = os.path.split(dst_path)
    if not os.path.exists(dst_path_head):
        raise FileNotFoundError('The specified destination directory '
                                'does not exist.')

    # If dst dir already exists ask user if it should be overwritten or not.
    if os.path.isdir(dst_path):
        warnings.warn(f'Folder "{dst_dirname}" already exists – '
                      'plot has already been published.')
        overwrite_dir = input('Do you want to overwrite the existing folder?'
                              ' (yes/no[default])\n')

        if overwrite_dir in ('yes', 'y'):
            shutil.rmtree(dst_path)
        else:
            raise RuntimeError('PlotID has finished without an export.\n'
                               'Rerun TagPlot if you need a new ID or '
                               'consider overwriting.')

    # Export plot figure to picture.
    plot_path = save_plot(figure, plot_name)

    # Create invisible folder
    dst_path_invisible = os.path.join(dst_path_head, '.' + dst_dirname)
    # If invisible Folder exists, delete it (publish was not successful before)
    if os.path.isdir(dst_path_invisible):
        shutil.rmtree(dst_path_invisible)

    match data_storage:
        case 'centralized':
            # Does nothing, not implemented yet
            pass
        case 'individual':
            # Copy all files to destination directory
            print('Copying data has been started. Depending on the size of '
                  'your data this may take a while...')
            # Copy data to invisible folder
            shutil.copytree(src_datapath, dst_path_invisible)

            # Copy script that calls this function to folder
            shutil.copy2(sys.argv[0], dst_path_invisible)
            # Copy plot file to folder
            if os.path.isfile(plot_path):
                shutil.copy2(plot_path, dst_path_invisible)
                os.remove(plot_path)
        case _:
            raise ValueError('The data storage method {data_storage} '
                             'is not available.')

    print('Copying data finished. Starting to clean up.')

    # If export was successful, make the directory visible
    os.rename(dst_path_invisible, dst_path)
    print(f'Publish was successful.\nYour plot "{plot_path}",\n'
          f'your data "{src_datapath}"\nand your script "{sys.argv[0]}"\n'
          f'were copied to {dst_path}\nin {data_storage} mode.')
